@@ -536,9 +536,7 @@ namespace gnote {
     const Note::Ptr self = std::static_pointer_cast<Note>(shared_from_this());
 
     if (!linking_notes.empty()) {
-      Glib::RefPtr<Gio::Settings> settings = m_gnote.preferences().get_schema_settings(Preferences::SCHEMA_GNOTE);
-      const NoteRenameBehavior behavior
-        = static_cast<NoteRenameBehavior>(settings->get_int(Preferences::NOTE_RENAME_BEHAVIOR));
+      const NoteRenameBehavior behavior = static_cast<NoteRenameBehavior>(m_gnote.preferences().note_rename_behavior());
 
       if (NOTE_RENAME_ALWAYS_SHOW_DIALOG == behavior) {
         NoteRenameDialog *dlg = new NoteRenameDialog(linking_notes, old_title, self, m_gnote);
@@ -574,8 +572,7 @@ namespace gnote {
       NoteRenameDialog *dlg = static_cast<NoteRenameDialog*>(dialog);
       const NoteRenameBehavior selected_behavior = dlg->get_selected_behavior();
       if(Gtk::RESPONSE_CANCEL != response && NOTE_RENAME_ALWAYS_SHOW_DIALOG != selected_behavior) {
-        Glib::RefPtr<Gio::Settings> settings = m_gnote.preferences().get_schema_settings(Preferences::SCHEMA_GNOTE);
-        settings->set_int(Preferences::NOTE_RENAME_BEHAVIOR, selected_behavior);
+        m_gnote.preferences().note_rename_behavior(selected_behavior);
       }
 
       const NoteRenameDialog::MapPtr notes = dlg->get_notes();
@@ -763,14 +760,13 @@ namespace gnote {
   }
 
   bool Note::is_special() const
-  { 
-    return manager().start_note_uri() == m_data.data().uri();
+  {
+    return m_gnote.preferences().start_note_uri() == m_data.data().uri();
   }
 
   bool Note::is_pinned() const
   {
-    Glib::ustring pinned_uris = m_gnote.preferences()
-      .get_schema_settings(Preferences::SCHEMA_GNOTE)->get_string(Preferences::MENU_PINNED_NOTES);
+    auto pinned_uris = m_gnote.preferences().menu_pinned_notes();
     return pinned_uris.find(uri()) != Glib::ustring::npos;
   }
 
@@ -778,8 +774,7 @@ namespace gnote {
   void Note::set_pinned(bool pinned) const
   {
     Glib::ustring new_pinned;
-    Glib::RefPtr<Gio::Settings> settings = m_gnote.preferences().get_schema_settings(Preferences::SCHEMA_GNOTE);
-    Glib::ustring old_pinned = settings->get_string(Preferences::MENU_PINNED_NOTES);
+    auto old_pinned = m_gnote.preferences().menu_pinned_notes();
     bool is_currently_pinned = old_pinned.find(uri()) != Glib::ustring::npos;
 
     if (pinned == is_currently_pinned)
@@ -797,7 +792,7 @@ namespace gnote {
         }
       }
     }
-    settings->set_string(Preferences::MENU_PINNED_NOTES, new_pinned);
+    m_gnote.preferences().menu_pinned_notes(new_pinned);
     m_gnote.notebook_manager().signal_note_pin_status_changed(*this, pinned);
   }
 
