@@ -1,7 +1,7 @@
 /*
  * gnote
  *
- * Copyright (C) 2010-2015,2017,2019 Aurimas Cernius
+ * Copyright (C) 2010-2015,2017,2019-2021 Aurimas Cernius
  * Copyright (C) 2009 Debarshi Ray
  * Copyright (C) 2009 Hubert Figuiere
  *
@@ -57,6 +57,9 @@
 
 #define DEFAULT_SYNC_CONFIGURED_CONFLICT_BEHAVIOR 0
 
+#define NEW_PROPERTY_EDITOR_BOOL(property, check) new sharp::PropertyEditorBool([this]()->bool { return m_gnote.preferences().property(); }, \
+          [this](bool v) { m_gnote.preferences().property(v); }, check);
+
 namespace gnote {
 
   struct CompareSyncAddinsByName
@@ -164,9 +167,10 @@ namespace gnote {
       add_action_widget (*button, Gtk::RESPONSE_CLOSE);
       set_default_response(Gtk::RESPONSE_CLOSE);
 
-    m_gnote.preferences().get_schema_settings(
-      Preferences::SCHEMA_GNOTE)->signal_changed().connect(
-        sigc::mem_fun(*this, &PreferencesDialog::on_preferences_setting_changed));
+    m_gnote.preferences().signal_note_rename_behavior_changed.connect(
+        sigc::mem_fun(*this, &PreferencesDialog::on_note_rename_behavior_changed));
+    m_gnote.preferences().signal_sync_autosync_timeout_changed
+      .connect(sigc::mem_fun(*this, &PreferencesDialog::on_autosync_timeout_setting_changed));
   }
 
   void PreferencesDialog::enable_addin(bool enable)
@@ -236,7 +240,6 @@ namespace gnote {
       Gtk::Label *label;
       Gtk::CheckButton *check;
       sharp::PropertyEditorBool *peditor, *font_peditor,* bullet_peditor;
-      Glib::RefPtr<Gio::Settings> settings = m_gnote.preferences().get_schema_settings(Preferences::SCHEMA_GNOTE);
 
       Gtk::Grid *options_list = manage(new Gtk::Grid);
       options_list->set_row_spacing(12);
@@ -248,7 +251,7 @@ namespace gnote {
       // Open in new window
       check = manage(make_check_button(_("Always _open notes in new window")));
       options_list->attach(*check, 0, options_list_row++, 1, 1);
-      peditor = new sharp::PropertyEditorBool(settings, Preferences::OPEN_NOTES_IN_NEW_WINDOW, *check);
+      peditor = NEW_PROPERTY_EDITOR_BOOL(open_notes_in_new_window, *check);
       peditor->setup();
 
 
@@ -263,7 +266,7 @@ namespace gnote {
         set_widget_tooltip(*check, _("Misspellings will be underlined in red, with correct spelling "
                                      "suggestions shown in the context menu."));
         options_list->attach(*check, 0, options_list_row++, 1, 1);
-        peditor = new sharp::PropertyEditorBool(settings, Preferences::ENABLE_SPELLCHECKING, *check);
+        peditor = NEW_PROPERTY_EDITOR_BOOL(enable_spellchecking, *check);
         peditor->setup();
       }
 #endif
@@ -273,8 +276,7 @@ namespace gnote {
       check = manage(make_check_button (_("Enable auto-_bulleted lists")));
       set_widget_tooltip(*check, _("Start new bulleted list by starting new line with character \"-\"."));
       options_list->attach(*check, 0, options_list_row++, 1, 1);
-      bullet_peditor = new sharp::PropertyEditorBool(settings, Preferences::ENABLE_AUTO_BULLETED_LISTS, 
-                                                       *check);
+      bullet_peditor = NEW_PROPERTY_EDITOR_BOOL(enable_auto_bulleted_lists, *check);
       bullet_peditor->setup();
 
       // Custom font...
@@ -283,8 +285,7 @@ namespace gnote {
       check = manage(make_check_button (_("Use custom _font")));
       check->set_hexpand(true);
       font_box->attach(*check, 0, 0, 1, 1);
-      font_peditor = new sharp::PropertyEditorBool(settings, Preferences::ENABLE_CUSTOM_FONT, 
-                                                     *check);
+      font_peditor = NEW_PROPERTY_EDITOR_BOOL(enable_custom_font, *check);
       font_peditor->setup();
 
       font_button = manage(make_font_button());
@@ -305,10 +306,10 @@ namespace gnote {
       m_rename_behavior_combo->append(_("Ask me what to do"));
       m_rename_behavior_combo->append(_("Never rename links"));
       m_rename_behavior_combo->append(_("Always rename links"));
-      int rename_behavior = settings->get_int(Preferences::NOTE_RENAME_BEHAVIOR);
+      int rename_behavior = m_gnote.preferences().note_rename_behavior();
       if (0 > rename_behavior || 2 < rename_behavior) {
         rename_behavior = 0;
-        settings->set_int(Preferences::NOTE_RENAME_BEHAVIOR, rename_behavior);
+        m_gnote.preferences().note_rename_behavior(rename_behavior);
       }
       m_rename_behavior_combo->set_active(rename_behavior);
       m_rename_behavior_combo->signal_changed().connect(
@@ -365,9 +366,7 @@ namespace gnote {
     button->add (*font_box);
     button->show ();
 
-    Glib::ustring font_desc = m_gnote.preferences().get_schema_settings(
-        Preferences::SCHEMA_GNOTE)->get_string(Preferences::CUSTOM_FONT_FACE);
-    update_font_button (font_desc);
+    update_font_button(m_gnote.preferences().custom_font_face());
 
     return button;
   }
@@ -382,14 +381,12 @@ namespace gnote {
     Gtk::CheckButton *check;
     sharp::PropertyEditorBool *peditor;
     int vbox_row = 0;
-    Glib::RefPtr<Gio::Settings> settings = m_gnote.preferences()
-      .get_schema_settings(Preferences::SCHEMA_GNOTE);
 
     // internal links
     check = manage(make_check_button(_("_Automatically link to notes")));
     set_widget_tooltip(*check, _("Enable this option to create a link when text matches note title."));
     vbox->attach(*check, 0, vbox_row++, 1, 1);
-    peditor = new sharp::PropertyEditorBool(settings, Preferences::ENABLE_AUTO_LINKS, *check);
+    peditor = NEW_PROPERTY_EDITOR_BOOL(enable_auto_links, *check);
     peditor->setup();
 
     // URLs
@@ -397,7 +394,7 @@ namespace gnote {
     set_widget_tooltip(*check, _("Enable this option to create links for URLs. "
                                  "Clicking will open URL with appropriate program."));
     vbox->attach(*check, 0, vbox_row++, 1, 1);
-    peditor = new sharp::PropertyEditorBool(settings, Preferences::ENABLE_URL_LINKS, *check);
+    peditor = NEW_PROPERTY_EDITOR_BOOL(enable_url_links, *check);
     peditor->setup();
 
     // WikiWords...
@@ -405,7 +402,7 @@ namespace gnote {
     set_widget_tooltip(*check, _("Enable this option to highlight words <b>ThatLookLikeThis</b>. "
                                  "Clicking the word will create a note with that name."));
     vbox->attach(*check, 0, vbox_row++, 1, 1);
-    peditor = new sharp::PropertyEditorBool(settings, Preferences::ENABLE_WIKIWORDS, *check);
+    peditor = NEW_PROPERTY_EDITOR_BOOL(enable_wikiwords, *check);
     peditor->setup();
 
     return vbox;
@@ -459,8 +456,7 @@ namespace gnote {
 
     // Read from Preferences which service is configured and select it
     // by default.  Otherwise, just select the first one in the list.
-    Glib::ustring addin_id = m_gnote.preferences()
-      .get_schema_settings(Preferences::SCHEMA_SYNC)->get_string(Preferences::SYNC_SELECTED_SERVICE_ADDIN);
+    Glib::ustring addin_id = m_gnote.preferences().sync_selected_service_addin();
 
     Gtk::TreeIter active_iter;
     if (!addin_id.empty() && m_sync_addin_iters.find(addin_id) != m_sync_addin_iters.end()) {
@@ -507,12 +503,10 @@ namespace gnote {
     vbox->attach(*m_sync_addin_prefs_container, 0, vbox_row++, 1, 1);
 
     // Autosync preference
-    int timeout = m_gnote.preferences().get_schema_settings(
-        Preferences::SCHEMA_SYNC)->get_int(Preferences::SYNC_AUTOSYNC_TIMEOUT);
+    int timeout = m_gnote.preferences().sync_autosync_timeout();
     if(timeout > 0 && timeout < 5) {
       timeout = 5;
-      m_gnote.preferences().get_schema_settings(
-          Preferences::SCHEMA_SYNC)->set_int(Preferences::SYNC_AUTOSYNC_TIMEOUT, 5);
+      m_gnote.preferences().sync_autosync_timeout(5);
     }
     Gtk::Grid *autosyncBox = manage(new Gtk::Grid);
     autosyncBox->set_column_spacing(5);
@@ -939,15 +933,14 @@ namespace gnote {
     Gtk::FontSelectionDialog *font_dialog =
       new Gtk::FontSelectionDialog (_("Choose Note Font"));
 
-    Glib::RefPtr<Gio::Settings> settings = m_gnote.preferences().get_schema_settings(Preferences::SCHEMA_GNOTE);
-    Glib::ustring font_name = settings->get_string(Preferences::CUSTOM_FONT_FACE);
+    auto font_name = m_gnote.preferences().custom_font_face();
     font_dialog->set_font_name(font_name);
 
     if (Gtk::RESPONSE_OK == font_dialog->run()) {
-      if (font_dialog->get_font_name() != font_name) {
-        settings->set_string(Preferences::CUSTOM_FONT_FACE, font_dialog->get_font_name());
-
-        update_font_button (font_dialog->get_font_name());
+      auto new_font = font_dialog->get_font_name();
+      if(new_font != font_name) {
+        m_gnote.preferences().custom_font_face(new_font);
+        update_font_button(new_font);
       }
     }
 
@@ -983,35 +976,33 @@ namespace gnote {
 
 
 
-  void  PreferencesDialog::on_preferences_setting_changed(const Glib::ustring & key)
+  void  PreferencesDialog::on_note_rename_behavior_changed()
   {
-    if (key == Preferences::NOTE_RENAME_BEHAVIOR) {
-      Glib::RefPtr<Gio::Settings> settings = m_gnote.preferences()
-        .get_schema_settings(Preferences::SCHEMA_GNOTE);
-      int rename_behavior = settings->get_int(key);
-      if (0 > rename_behavior || 2 < rename_behavior) {
-        rename_behavior = 0;
-        settings->set_int(Preferences::NOTE_RENAME_BEHAVIOR, rename_behavior);
-      }
-      if (m_rename_behavior_combo->get_active_row_number()
-            != rename_behavior) {
-        m_rename_behavior_combo->set_active(rename_behavior);
-      }
+    int rename_behavior = m_gnote.preferences().note_rename_behavior();
+    if(0 > rename_behavior || 2 < rename_behavior) {
+      rename_behavior = 0;
+      m_gnote.preferences().note_rename_behavior(rename_behavior);
     }
-    else if(key == Preferences::SYNC_AUTOSYNC_TIMEOUT) {
-      int timeout = m_gnote.preferences().get_schema_settings(
-          Preferences::SCHEMA_SYNC)->get_int(Preferences::SYNC_AUTOSYNC_TIMEOUT);
-      if(timeout <= 0 && m_autosync_check->get_active()) {
-        m_autosync_check->set_active(false);
+    if(m_rename_behavior_combo->get_active_row_number() != rename_behavior) {
+      m_rename_behavior_combo->set_active(rename_behavior);
+    }
+  }
+
+
+
+  void PreferencesDialog::on_autosync_timeout_setting_changed()
+  {
+    int timeout = m_gnote.preferences().sync_autosync_timeout();
+    if(timeout <= 0 && m_autosync_check->get_active()) {
+      m_autosync_check->set_active(false);
+    }
+    else if(timeout > 0) {
+      timeout = (timeout >= 5 && timeout < 1000) ? timeout : 5;
+      if(!m_autosync_check->get_active()) {
+        m_autosync_check->set_active(true);
       }
-      else if(timeout > 0) {
-        timeout = (timeout >= 5 && timeout < 1000) ? timeout : 5;
-        if(!m_autosync_check->get_active()) {
-          m_autosync_check->set_active(true);
-        }
-        if(static_cast<int>(m_autosync_spinner->get_value()) != timeout) {
-          m_autosync_spinner->set_value(timeout);
-        }
+      if(static_cast<int>(m_autosync_spinner->get_value()) != timeout) {
+        m_autosync_spinner->set_value(timeout);
       }
     }
   }
@@ -1020,8 +1011,7 @@ namespace gnote {
 
   void  PreferencesDialog::on_rename_behavior_changed()
   {
-    m_gnote.preferences().get_schema_settings(Preferences::SCHEMA_GNOTE)->set_int(
-        Preferences::NOTE_RENAME_BEHAVIOR, m_rename_behavior_combo->get_active_row_number());
+    m_gnote.preferences().note_rename_behavior(m_rename_behavior_combo->get_active_row_number());
   }
 
 
@@ -1029,8 +1019,7 @@ namespace gnote {
   {
     // Get saved behavior
     sync::SyncTitleConflictResolution savedBehavior = sync::CANCEL;
-    int dlgBehaviorPref = m_gnote.preferences().get_schema_settings(
-      Preferences::SCHEMA_SYNC)->get_int(Preferences::SYNC_CONFIGURED_CONFLICT_BEHAVIOR);
+    int dlgBehaviorPref = m_gnote.preferences().sync_configured_conflict_behavior();
     // TODO: Check range of this int
     savedBehavior = static_cast<sync::SyncTitleConflictResolution>(dlgBehaviorPref);
 
@@ -1096,8 +1085,7 @@ namespace gnote {
       newBehavior = sync::OVERWRITE_EXISTING;
     }
 
-    m_gnote.preferences().get_schema_settings(Preferences::SCHEMA_SYNC)->set_int(
-      Preferences::SYNC_CONFIGURED_CONFLICT_BEHAVIOR, static_cast<int>(newBehavior));
+    m_gnote.preferences().sync_configured_conflict_behavior(static_cast<int>(newBehavior));
   }
 
 
@@ -1178,11 +1166,10 @@ namespace gnote {
       DBG_OUT("Error calling %s.reset_configuration: %s", m_selected_sync_addin->id().c_str(), e.what());
     }
 
-    Glib::RefPtr<Gio::Settings> settings = m_gnote.preferences().get_schema_settings(Preferences::SCHEMA_SYNC);
-    settings->set_string(Preferences::SYNC_SELECTED_SERVICE_ADDIN, "");
+    m_gnote.preferences().sync_selected_service_addin("");
 
     // Reset conflict handling behavior
-    settings->set_int(Preferences::SYNC_CONFIGURED_CONFLICT_BEHAVIOR, DEFAULT_SYNC_CONFIGURED_CONFLICT_BEHAVIOR);
+    m_gnote.preferences().sync_configured_conflict_behavior(DEFAULT_SYNC_CONFIGURED_CONFLICT_BEHAVIOR);
 
     m_gnote.sync_manager().reset_client();
 
@@ -1227,8 +1214,7 @@ namespace gnote {
 
     utils::HIGMessageDialog *dialog;
     if(saved) {
-      m_gnote.preferences().get_schema_settings(Preferences::SCHEMA_SYNC)->set_string(
-        Preferences::SYNC_SELECTED_SERVICE_ADDIN, m_selected_sync_addin->id());
+      m_gnote.preferences().sync_selected_service_addin(m_selected_sync_addin->id());
 
       m_sync_addin_combo->set_sensitive(false);
       m_sync_addin_prefs_widget->set_sensitive(false);
@@ -1248,15 +1234,15 @@ namespace gnote {
 
       if(dialog_response == Gtk::RESPONSE_YES) {
         // TODO: Put this voodoo in a method somewhere
-        m_gnote.action_manager().get_app_action("sync-notes")->activate(Glib::VariantBase());
+        auto action = m_gnote.action_manager().get_app_action("sync-notes");
+        utils::main_context_invoke([action = std::move(action)]() { action->activate(Glib::VariantBase()); });
       }
     }
     else {
       // TODO: Change the SyncServiceAddin API so the call to
       // SaveConfiguration has a way of passing back an exception
       // or other text so it can be displayed to the user.
-      m_gnote.preferences().get_schema_settings(Preferences::SCHEMA_SYNC)->set_string(
-       Preferences::SYNC_SELECTED_SERVICE_ADDIN, "");
+      m_gnote.preferences().sync_selected_service_addin("");
 
       m_sync_addin_combo->set_sensitive(true);
       m_sync_addin_prefs_widget->set_sensitive(true);
@@ -1429,8 +1415,7 @@ namespace gnote {
 
   void PreferencesDialog::update_timeout_pref()
   {
-    m_gnote.preferences().get_schema_settings(Preferences::SCHEMA_SYNC)->set_int(
-        Preferences::SYNC_AUTOSYNC_TIMEOUT,
+    m_gnote.preferences().sync_autosync_timeout(
         m_autosync_check->get_active() ? static_cast<int>(m_autosync_spinner->get_value()) : -1);
   }
 
