@@ -1,7 +1,7 @@
 /*
  * gnote
  *
- * Copyright (C) 2010-2019 Aurimas Cernius
+ * Copyright (C) 2010-2021 Aurimas Cernius
  * Copyright (C) 2010 Debarshi Ray
  * Copyright (C) 2009 Hubert Figuiere
  *
@@ -53,52 +53,49 @@ namespace gnote {
     , m_gnote(g)
     , m_note_manager(m)
     , m_preferences(g.preferences())
-    , m_search_notes_widget(g, m)
-    , m_search_box(0.5, 0.5, 0.0, 1.0)
+    , m_search_box(nullptr)
+    , m_find_next_prev_box(nullptr)
+    , m_search_entry(nullptr)
     , m_mapped(false)
     , m_entry_changed_timeout(NULL)
     , m_window_menu_embedded(NULL)
     , m_window_menu_default(NULL)
     , m_keybinder(get_accel_group())
   {
-    Glib::RefPtr<Gio::Settings> settings = g.preferences().get_schema_settings(Preferences::SCHEMA_GNOTE);
-    m_open_notes_in_new_window = settings->get_boolean(Preferences::OPEN_NOTES_IN_NEW_WINDOW);
-    m_close_note_on_escape = settings->get_boolean(Preferences::ENABLE_CLOSE_NOTE_ON_ESCAPE);
     set_default_size(450,400);
     set_resizable(true);
-    if(settings->get_boolean(Preferences::MAIN_WINDOW_MAXIMIZED)) {
+    if(g.preferences().main_window_maximized()) {
       maximize();
     }
-    settings->signal_changed().connect(sigc::mem_fun(*this, &NoteRecentChanges::on_settings_changed));
 
     set_has_resize_grip(true);
     set_icon_name(IconManager::GNOTE);
 
-    m_search_notes_widget.signal_open_note
+    m_search_notes_widget = new SearchNotesWidget(g, m);
+    m_search_notes_widget->signal_open_note
       .connect(sigc::mem_fun(*this, &NoteRecentChanges::on_open_note));
-    m_search_notes_widget.signal_open_note_new_window
+    m_search_notes_widget->signal_open_note_new_window
       .connect(sigc::mem_fun(*this, &NoteRecentChanges::on_open_note_new_window));
-    m_search_notes_widget.notes_widget().signal_key_press_event()
+    m_search_notes_widget->notes_widget().signal_key_press_event()
       .connect(sigc::mem_fun(*this, &NoteRecentChanges::on_notes_widget_key_press));
 
     make_header_bar();
-    make_search_box();
-    m_content_vbox.set_orientation(Gtk::ORIENTATION_VERTICAL);
+    auto content = manage(new Gtk::Grid);
+    content->set_orientation(Gtk::ORIENTATION_VERTICAL);
     int content_y_attach = 0;
     if(use_client_side_decorations(m_preferences)) {
       set_titlebar(*static_cast<Gtk::HeaderBar*>(m_header_bar));
     }
     else {
-      m_content_vbox.attach(*m_header_bar, 0, content_y_attach++, 1, 1);
+      content->attach(*m_header_bar, 0, content_y_attach++, 1, 1);
     }
-    m_content_vbox.attach(m_search_box, 0, content_y_attach++, 1, 1);
-    m_content_vbox.attach(m_embed_box, 0, content_y_attach++, 1, 1);
+    content->attach(m_embed_box, 0, content_y_attach++, 1, 1);
     m_embed_box.set_hexpand(true);
     m_embed_box.set_vexpand(true);
     m_embed_box.show();
-    m_content_vbox.show ();
+    content->show();
 
-    add (m_content_vbox);
+    add(*content);
     signal_delete_event().connect(sigc::mem_fun(*this, &NoteRecentChanges::on_delete));
     signal_key_press_event()
       .connect(sigc::mem_fun(*this, &NoteRecentChanges::on_key_pressed));
@@ -132,18 +129,19 @@ namespace gnote {
     find_action("close-window")->signal_activate()
       .connect(sigc::mem_fun(*this, &NoteRecentChanges::on_close_window));
 
-    embed_widget(m_search_notes_widget);
+    embed_widget(*m_search_notes_widget);
   }
 
 
   NoteRecentChanges::~NoteRecentChanges()
   {
-    while(m_embedded_widgets.size()) {
-      unembed_widget(**m_embedded_widgets.begin());
-    }
     if(m_entry_changed_timeout) {
       delete m_entry_changed_timeout;
     }
+    if(!m_search_box && m_search_text) {
+      delete m_search_text;
+    }
+    delete m_search_notes_widget;
   }
 
   void NoteRecentChanges::make_header_bar()
@@ -170,7 +168,7 @@ namespace gnote {
     m_new_note_button->set_image(*image);
     m_new_note_button->set_tooltip_text(_("Create New Note"));
     m_new_note_button->add_accelerator("activate", get_accel_group(), GDK_KEY_N, Gdk::CONTROL_MASK, (Gtk::AccelFlags) 0);
-    m_new_note_button->signal_clicked().connect(sigc::mem_fun(m_search_notes_widget, &SearchNotesWidget::new_note));
+    m_new_note_button->signal_clicked().connect(sigc::mem_fun(*m_search_notes_widget, &SearchNotesWidget::new_note));
     m_new_note_button->show_all();
     left_box->attach(*m_new_note_button, 1, 0, 1, 1);
     left_box->show();
@@ -232,47 +230,87 @@ namespace gnote {
 
   void NoteRecentChanges::make_search_box()
   {
-    m_search_entry.set_activates_default(false);
-    m_search_entry.set_size_request(300);
-    m_search_entry.signal_key_press_event()
+    if(m_search_box) {
+      return;
+    }
+
+    Glib::ustring search_text;
+    if(m_search_text) {
+      search_text = *m_search_text;
+      delete m_search_text;
+    }
+    m_search_entry = manage(new Gtk::SearchEntry);
+    m_search_entry->set_text(search_text);
+    m_search_entry->set_activates_default(false);
+    m_search_entry->set_size_request(300);
+    m_search_entry->signal_key_press_event()
       .connect(sigc::mem_fun(*this, &NoteRecentChanges::on_entry_key_pressed), false);
-    m_search_entry.signal_changed()
+    m_search_entry->signal_changed()
       .connect(sigc::mem_fun(*this, &NoteRecentChanges::on_entry_changed));
-    m_search_entry.signal_activate()
+    m_search_entry->signal_activate()
       .connect(sigc::mem_fun(*this, &NoteRecentChanges::on_entry_activated));
-    m_search_entry.show();
+    m_search_entry->show();
 
     Gtk::Grid *grid = manage(new Gtk::Grid);
     grid->set_margin_left(5);
     grid->set_margin_right(5);
     grid->set_hexpand(false);
-    grid->attach(m_search_entry, 0, 0, 1, 1);
+    grid->attach(*m_search_entry, 0, 0, 1, 1);
+    grid->show();
 
-    m_find_next_prev_box.set_margin_left(5);
+    m_search_box = manage(new Gtk::Alignment(0.5, 0.5, 0.0, 1.0));
+    m_search_box->add(*grid);
+    m_search_box->set_hexpand(true);
 
-    Gtk::Button *find_next_button = manage(new Gtk::Button(_("Find _Next"), true));
-    find_next_button->set_image(*manage(new Gtk::Image(Gtk::Stock::GO_FORWARD, Gtk::ICON_SIZE_MENU)));
+    auto content = dynamic_cast<Gtk::Grid*>(m_embed_box.get_parent());
+    if(content) {
+      content->attach_next_to(*m_search_box, m_embed_box, Gtk::POS_TOP);
+    }
+    else {
+      ERR_OUT(_("Parent of embed box is not a Gtk::Grid, please report a bug!"));
+    }
+  }
+
+  void NoteRecentChanges::make_find_next_prev()
+  {
+    if(m_find_next_prev_box) {
+      return;
+    }
+
+    m_find_next_prev_box = manage(new Gtk::Grid);
+    m_find_next_prev_box->set_margin_left(5);
+
+    Gtk::Button *find_next_button = manage(new Gtk::Button);
+    Gtk::Image *image = manage(new Gtk::Image);
+    image->property_icon_name() = "go-down-symbolic";
+    image->property_icon_size() = GTK_ICON_SIZE_MENU;
+    find_next_button->set_image(*image);
     find_next_button->set_always_show_image(true);
     find_next_button->signal_clicked()
       .connect(sigc::mem_fun(*this, &NoteRecentChanges::on_find_next_button_clicked));
-    find_next_button->add_accelerator("activate", get_accel_group(), GDK_KEY_Return, (Gdk::ModifierType)0, (Gtk::AccelFlags) 0);
+    find_next_button->add_accelerator("activate", get_accel_group(), GDK_KEY_G, Gdk::CONTROL_MASK, (Gtk::AccelFlags) 0);
     find_next_button->show();
-    m_find_next_prev_box.attach(*find_next_button, 0, 0, 1, 1);
+    m_find_next_prev_box->attach(*find_next_button, 0, 0, 1, 1);
 
-    Gtk::Button *find_prev_button = manage(new Gtk::Button(_("Find _Previous"), true));
-    find_prev_button->set_image(*manage(new Gtk::Image(Gtk::Stock::GO_BACK, Gtk::ICON_SIZE_MENU)));
+    Gtk::Button *find_prev_button = manage(new Gtk::Button);
+    image = manage(new Gtk::Image);
+    image->property_icon_name() = "go-up-symbolic";
+    image->property_icon_size() = GTK_ICON_SIZE_MENU;
+    find_prev_button->set_image(*image);
     find_prev_button->set_always_show_image(true);
     find_prev_button->signal_clicked()
       .connect(sigc::mem_fun(*this, &NoteRecentChanges::on_find_prev_button_clicked));
-    find_prev_button->add_accelerator("activate", get_accel_group(), GDK_KEY_Return, Gdk::SHIFT_MASK, (Gtk::AccelFlags) 0);
+    find_prev_button->add_accelerator("activate", get_accel_group(), GDK_KEY_G, Gdk::CONTROL_MASK|Gdk::SHIFT_MASK, (Gtk::AccelFlags) 0);
     find_prev_button->show();
-    m_find_next_prev_box.attach(*find_prev_button, 1, 0, 1, 1);
+    m_find_next_prev_box->attach(*find_prev_button, 1, 0, 1, 1);
 
-    grid->attach(m_find_next_prev_box, 1, 0, 1, 1);
-    grid->show();
-
-    m_search_box.add(*grid);
-    m_search_box.set_hexpand(true);
+    auto grid = dynamic_cast<Gtk::Grid*>(m_search_entry->get_parent());
+    if(grid) {
+      grid->attach(*m_find_next_prev_box, 1, 0, 1, 1);
+    }
+    else {
+      ERR_OUT(_("Parent of search entry is not Gtk::Grid, please report a bug!"));
+    }
   }
 
   void NoteRecentChanges::on_search_button_toggled()
@@ -281,7 +319,7 @@ namespace gnote {
       show_search_bar();
     }
     else {
-      m_search_box.hide();
+      m_search_box->hide();
       SearchableItem *searchable_widget = dynamic_cast<SearchableItem*>(currently_embedded());
       if(searchable_widget) {
         searchable_widget->perform_search("");
@@ -307,32 +345,54 @@ namespace gnote {
 
   void NoteRecentChanges::show_search_bar(bool focus)
   {
-    if(m_search_box.get_visible()) {
-      return;
+    make_search_box();
+    if(m_search_box->get_visible()) {
+      focus = false;
     }
-    m_search_box.show();
+    m_search_box->show();
     if(focus) {
-      m_search_entry.grab_focus();
+      m_search_entry->grab_focus();
     }
-    Glib::ustring text = m_search_entry.get_text();
-    if(text != "") {
-      SearchableItem *searchable_widget = dynamic_cast<SearchableItem*>(currently_embedded());
-      if(searchable_widget) {
-        searchable_widget->perform_search(text);
+    Glib::ustring text = m_search_entry->get_text();
+    update_search_bar(*currently_embedded(), text != "");
+  }
+
+  void NoteRecentChanges::update_search_bar(EmbeddableWidget & widget, bool perform_search)
+  {
+    SearchableItem *searchable_item = dynamic_cast<SearchableItem*>(&widget);
+    if(searchable_item) {
+      m_search_button.show();
+      if(searchable_item->supports_goto_result()) {
+        if(m_search_box && m_search_box->get_visible()) {
+          make_find_next_prev();
+          m_find_next_prev_box->show();
+        }
       }
+      else {
+        if(m_find_next_prev_box) {
+          m_find_next_prev_box->hide();
+        }
+      }
+      if(perform_search) {
+        searchable_item->perform_search(m_search_button.get_active() ? m_search_entry->get_text() : "");
+      }
+    }
+    else {
+      m_search_button.set_active(false);
+      m_search_button.hide();
     }
   }
 
   void NoteRecentChanges::present_search()
   {
     EmbeddableWidget *current = currently_embedded();
-    if(&m_search_notes_widget == dynamic_cast<SearchNotesWidget*>(current)) {
+    if(m_search_notes_widget == dynamic_cast<SearchNotesWidget*>(current)) {
       return;
     }
     if(current) {
       background_embedded(*current);
     }
-    foreground_embedded(m_search_notes_widget);
+    foreground_embedded(*m_search_notes_widget);
   }
 
   void NoteRecentChanges::present_note(const Note::Ptr & note)
@@ -356,7 +416,7 @@ namespace gnote {
 
   void NoteRecentChanges::on_open_note(const Note::Ptr & note)
   {
-    if(m_open_notes_in_new_window) {
+    if(m_preferences.open_notes_in_new_window()) {
       on_open_note_new_window(note);
     }
     else {
@@ -368,12 +428,12 @@ namespace gnote {
 
   void NoteRecentChanges::on_open_note_new_window(const Note::Ptr & note)
   {
-    present_in_new_window(m_gnote, note, m_close_note_on_escape);
+    present_in_new_window(m_gnote, note, m_preferences.enable_close_note_on_escape());
   }
 
   void NoteRecentChanges::on_delete_note()
   {
-    m_search_notes_widget.delete_selected_notes();
+    m_search_notes_widget->delete_selected_notes();
   }
 
 
@@ -383,9 +443,7 @@ namespace gnote {
     Glib::RefPtr<Gdk::Window> win = get_window();
     // background window (for tray to work) might not have GDK window
     if(win) {
-      m_preferences.get_schema_settings(Preferences::SCHEMA_GNOTE)->set_boolean(
-          Preferences::MAIN_WINDOW_MAXIMIZED,
-          win->get_state() & Gdk::WINDOW_STATE_MAXIMIZED);
+      m_preferences.main_window_maximized(win->get_state() & Gdk::WINDOW_STATE_MAXIMIZED);
     }
     std::vector<Gtk::Widget*> current = m_embed_box.get_children();
     for(std::vector<Gtk::Widget*>::iterator iter = current.begin();
@@ -402,7 +460,7 @@ namespace gnote {
 
   bool NoteRecentChanges::is_search()
   {
-    return &m_search_notes_widget == currently_embedded();
+    return m_search_notes_widget == currently_embedded();
   }
 
 
@@ -418,24 +476,26 @@ namespace gnote {
     return true;
   }
 
-  bool NoteRecentChanges::on_key_pressed(GdkEventKey * ev)
+  bool NoteRecentChanges::on_key_pressed(GdkEventKey *ev)
   {
-    switch (ev->keyval) {
+    guint keyval;
+    gdk_event_get_keyval((GdkEvent*)ev, &keyval);
+    switch(keyval) {
     case GDK_KEY_Escape:
       if(m_search_button.get_active()) {
-        m_search_entry.set_text("");
+        m_search_entry->set_text("");
         m_search_button.set_active(false);
       }
       // Allow Escape to close the window
       else if(close_on_escape()) {
         close_window();
       }
-      else if(m_close_note_on_escape) {
+      else if(m_preferences.enable_close_note_on_escape()) {
         EmbeddableWidget *current_item = currently_embedded();
         if(current_item) {
           background_embedded(*current_item);
         }
-        foreground_embedded(m_search_notes_widget);
+        foreground_embedded(*m_search_notes_widget);
       }
       break;
     case GDK_KEY_F1:
@@ -451,7 +511,7 @@ namespace gnote {
   void NoteRecentChanges::on_show()
   {
     // Select "All Notes" in the notebooks list
-    m_search_notes_widget.select_all_notes_notebook();
+    m_search_notes_widget->select_all_notes_notebook();
 
     EmbeddableWidget *widget = NULL;
     if(m_embed_box.get_children().size() == 0 && m_embedded_widgets.size() > 0) {
@@ -472,7 +532,17 @@ namespace gnote {
 
   void NoteRecentChanges::set_search_text(const Glib::ustring & value)
   {
-    m_search_entry.set_text(value);
+    if(m_search_box) {
+      m_search_entry->set_text(value);
+    }
+    else {
+      if(!m_search_text) {
+        m_search_text = new Glib::ustring(value);
+      }
+      else {
+        *m_search_text = value;
+      }
+    }
   }
 
   void NoteRecentChanges::embed_widget(EmbeddableWidget & widget)
@@ -527,8 +597,7 @@ namespace gnote {
       wid.show();
       widget.foreground();
 
-      bool maximized = m_preferences.get_schema_settings(Preferences::SCHEMA_GNOTE)->get_boolean(
-        Preferences::MAIN_WINDOW_MAXIMIZED);
+      bool maximized = m_preferences.main_window_maximized();
       if(get_realized()) {
         //if window is showing, use actual state
         maximized = get_window()->get_state() & Gdk::WINDOW_STATE_MAXIMIZED;
@@ -544,7 +613,7 @@ namespace gnote {
       widget.size_internals();
  
       update_toolbar(widget);
-      if(&widget == &m_search_notes_widget) {
+      if(&widget == m_search_notes_widget) {
         set_title(_("Gnote"));
       }
       else {
@@ -645,15 +714,23 @@ namespace gnote {
   bool NoteRecentChanges::on_map_event(GdkEventAny *evt)
   {
     bool res = MainWindow::on_map_event(evt);
+    if(!m_mapped) {
+      auto widget = currently_embedded();
+      if(widget) {
+        widget->set_initial_focus();
+      }
+    }
     m_mapped = true;
     return res;
   }
 
-  bool NoteRecentChanges::on_entry_key_pressed(GdkEventKey *event)
+  bool NoteRecentChanges::on_entry_key_pressed(GdkEventKey *ev)
   {
-    switch(event->keyval) {
+    guint keyval;
+    gdk_event_get_keyval((GdkEvent*)ev, &keyval);
+    switch(keyval) {
     case GDK_KEY_Escape:
-      m_search_entry.set_text("");
+      m_search_entry->set_text("");
       m_search_button.set_active(false);
     }
 
@@ -662,7 +739,7 @@ namespace gnote {
 
   void NoteRecentChanges::on_entry_changed()
   {
-    if(!m_search_box.get_visible()) {
+    if(!m_search_box || !m_search_box->get_visible()) {
       return;
     }
     if(m_entry_changed_timeout == NULL) {
@@ -694,7 +771,7 @@ namespace gnote {
 
   void NoteRecentChanges::entry_changed_timeout()
   {
-    if(!m_search_box.get_visible()) {
+    if(!m_search_box || !m_search_box->get_visible()) {
       return;
     }
     Glib::ustring search_text = get_search_text();
@@ -710,33 +787,24 @@ namespace gnote {
 
   Glib::ustring NoteRecentChanges::get_search_text()
   {
-    Glib::ustring text = m_search_entry.get_text();
+    Glib::ustring text;
+    if(m_search_box) {
+      text = m_search_entry->get_text();
+    }
+    else if(m_search_text) {
+      text = *m_search_text;
+    }
     text = sharp::string_trim(text);
     return text;
   }
 
   void NoteRecentChanges::update_toolbar(EmbeddableWidget & widget)
   {
-    bool search = dynamic_cast<SearchNotesWidget*>(&widget) == &m_search_notes_widget;
+    bool search = dynamic_cast<SearchNotesWidget*>(&widget) == m_search_notes_widget;
     m_all_notes_button->set_visible(!search);
     m_new_note_button->set_visible(search);
     dynamic_cast<Gtk::Image*>(m_window_actions_button->get_image())->property_icon_name() = search ? MAIN_MENU_PRIMARY_ICON : MAIN_MENU_SECONDARY_ICON;
-
-    try {
-      SearchableItem & searchable_item = dynamic_cast<SearchableItem&>(widget);
-      m_search_button.show();
-      if(searchable_item.supports_goto_result()) {
-        m_find_next_prev_box.show();
-      }
-      else {
-        m_find_next_prev_box.hide();
-      }
-      searchable_item.perform_search(m_search_button.get_active() ? m_search_entry.get_text() : "");
-    }
-    catch(std::bad_cast &) {
-      m_search_button.set_active(false);
-      m_search_button.hide();
-    }
+    update_search_bar(widget, true);
 
     try {
       HasEmbeddableToolbar & toolbar_provider = dynamic_cast<HasEmbeddableToolbar&>(widget);
@@ -824,36 +892,26 @@ namespace gnote {
     }
   }
 
-  void NoteRecentChanges::on_settings_changed(const Glib::ustring & key)
+  bool NoteRecentChanges::on_notes_widget_key_press(GdkEventKey *ev)
   {
-    if(key == Preferences::OPEN_NOTES_IN_NEW_WINDOW) {
-      m_open_notes_in_new_window = m_preferences.get_schema_settings(
-        Preferences::SCHEMA_GNOTE)->get_boolean(Preferences::OPEN_NOTES_IN_NEW_WINDOW);
-    }
-    else if(key == Preferences::ENABLE_CLOSE_NOTE_ON_ESCAPE) {
-      m_close_note_on_escape = m_preferences.get_schema_settings(
-        Preferences::SCHEMA_GNOTE)->get_boolean(Preferences::ENABLE_CLOSE_NOTE_ON_ESCAPE);
-    }
-  }
-
-  bool NoteRecentChanges::on_notes_widget_key_press(GdkEventKey *evt)
-  {
-    switch(evt->keyval) {
+    guint keyval;
+    gdk_event_get_keyval((GdkEvent*)ev, &keyval);
+    switch(keyval) {
     case GDK_KEY_Escape:
     case GDK_KEY_Delete:
     case GDK_KEY_Tab:
       return false;
     case GDK_KEY_BackSpace:
       if(m_search_button.get_active()) {
-        Glib::ustring s = m_search_entry.get_text();
+        Glib::ustring s = m_search_entry->get_text();
         if(s.size()) {
-          m_search_entry.set_text(s.substr(0, s.size() - 1));
+          m_search_entry->set_text(s.substr(0, s.size() - 1));
         }
       }
       return false;
     default:
       {
-        guint32 character = gdk_keyval_to_unicode(evt->keyval);
+        guint32 character = gdk_keyval_to_unicode(keyval);
         if(character) {  // ignore special keys
           if(!m_search_button.get_active()) {
             // first show search box, then activate button
@@ -863,7 +921,7 @@ namespace gnote {
           }
           Glib::ustring s;
           s += character;
-          g_signal_emit_by_name(m_search_entry.gobj(), "insert-at-cursor", s.c_str());
+          g_signal_emit_by_name(m_search_entry->gobj(), "insert-at-cursor", s.c_str());
           return true;
         }
         return false;
